/**
 * Copyright 2010 JBoss Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.amentra.brms;

import java.util.HashMap;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.WorkflowProcessInstance;
//import org.infinispan.Cache;

import com.amentra.brms.SendTaskHandler;
import com.redhat.messaging.AbstractMessageSender;
import com.redhat.messaging.JMSMessageSender;



public class RemoteTrigger1Tests implements MessageConstants
{
	public RemoteTrigger1Tests()
	{
		super();
	}

    public void testServiceTask() throws Exception {
        KnowledgeBase kbase = createKnowledgeBase();
		StatefulKnowledgeSession ksession = createKnowledgeSession(kbase);
        ksession.getWorkItemManager().registerWorkItemHandler("SendService", new SendTaskHandler());
        //ksession.getWorkItemManager().registerWorkItemHandler("ReceiveService", new ReceiveTaskHandler());
        
        long start = System.currentTimeMillis();
               
        StringBuilder messageID = new StringBuilder("SendService.message");
        int messageIDl = messageID.length();
        
		HashMap<String, Object> hashMap = new HashMap<String, Object>();
        StringBuilder str = new StringBuilder("kris in jbpm example"); 
        int org = str.length();
		hashMap.put("brms.processID", "com.boa.bpm.COE");

		hashMap.put("brms.taskName", "BOA Task 2");
		hashMap.put("brms.taskTimeLimit", new Long(2000));  // seconds
    	
    	hashMap.put("messageID", messageID);
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("destination1", "jms/queue/requestQueue1");  
        params.put("message1", hashMap);
    	
        System.out.println("Start."); 
        //System.out.println(params.toString());
        int count = 100;
        int offset = 1000;
        int index = 0;
        for (int i = 0; i < count; i++) {
        	str.setLength(org);
        	str.append(i);
        	
        	index = offset + i;
    		hashMap.put("brms.processInstanceID", new Long(index));
    		hashMap.put("brms.sessionID", new Integer(index));
    		hashMap.put("brms.taskID", new Long(index));
        	
        	messageID.setLength(messageIDl);
        	messageID.append(i);
        	params.put("messageID1", messageID.toString());

	        WorkflowProcessInstance processInstance = (WorkflowProcessInstance)
	            ksession.startProcess("com.boa.brms.bpmn", params);
        }
        
        start = System.currentTimeMillis() - start;
        double ave = (double) start / (double) count;
        System.out.println("Total count: " + count);
        System.out.println("Average Time(ms): " + ave);
    }
	
	private KnowledgeBase createKnowledgeBase() throws Exception {
		System.out.println("Loading processes."); 
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("bpmn2/remoteTrigger1.bpmn"), ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}
	
	private StatefulKnowledgeSession createKnowledgeSession(KnowledgeBase kbase) {
		return kbase.newStatefulKnowledgeSession();
	}

	public static void main(String [] args)
	{
		try {
			RemoteTrigger1Tests test = new RemoteTrigger1Tests();
			AbstractMessageSender messageSender = JMSMessageSender.getInstance();
			HashMap<String, Object> hashMap = new HashMap<String, Object>();

			hashMap.put("brms.processID", "com.boa.bpm.COE");
			hashMap.put("brms.processInstanceID", new Long(1000));
			hashMap.put("brms.sessionID", new Integer(1000));
			hashMap.put("brms.taskID", new Long(1000));
			hashMap.put("brms.taskName", "BOA Task 1000");
			hashMap.put("brms.taskTimeLimit", new Long(2000)); 
			
			messageSender.sendMessage("jms/queue/requestQueue1", hashMap);
			
			test.testServiceTask();
			
			JMSMessageSender.getInstance().stop();	

			try {
				Thread.sleep(2000L);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			System.out.println("Clean up completed.");
			System.exit(0);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
